unit EuroPSD;

{******************************************
*
*             ensemble de procedures EuroPSD
version2
* E Leymarie  05/00
10/10: modification pour utilisation du cable dcal de 3 bits
{ MODIFICATIONS                                                                     }
{ mise a jour le 31.10.2000                                                         }
{ 21.11.2000: EL : modif de litspectre2D_PAPOL
{                  suppression de litspectre2D_2_PAPOL
{ ***********************************************}
{11/12: Edouard : modification des procedures d'initialisation en vue de l'utilisation de l'EUROPSD modifie}
{= creation de param_TOF_var et EuroPSDInitTOF_Var}
{ ***********************************************}
{ 14.12.2000: FO : deux nouvelles procdures
   procedure litspectre_PRISM(commande:string;moniteur:real;indexspectre,indexspectre1D:integer;comment:string);
   procedure litspectre2D_PRISM(moniteur:real;indexspectre,indexspectre1D:integer;comment:string;polar:string); }
{**************************************************}
{ 17/01/2001 : EL : modification des procedures de lecture des spectres PAPOL pour inclusion du moniteur}
{ 26.01.2001 : FO : configuration de l'EuroPSD pour faire du temps de vol     }
{                   Procedure EuroPSDInitTOF_EROS(retard,largeurcanal:real;nbrframe:integer); }
{27.03.01 : EL:modifications pour jumelage avec traitement:
               modification data[x,y] en data[x,y,0]
{13.06.01: EL :modifications pour enregistrement du moniteur sur spectre.moniteur}
{17.09.01 : EL :debut des modifications pour manip d'octobre : acquisition TOF sauvegarde en 3D
                cration de litspectre_TOF32_PAPOL

{09.10.01 : EL : pour une acquisition a frame de duree variable (EUROPSD custom), le step est pass par constante
                periodeframe=10000 pour 1 ms}
{13.12.01 : FO : * nouvelle procedure pour faire l'acquisition sur l'EuroPSD avec compteur Gabriel }
{                  procedure litspectre1D_Gabriel(moniteur:real;indexspectre1D:integer;comment:string;polar:string); }
{                * decalage de 1 octet pour la lecture de l'EiroPSD avec compteur Gabriel                           }
{                  'T16257,16384'                                                                                   }
{05.02.03: FO :  }
{version a jour le 09.10.01}
{ =================================== }
{ A FAIRE                             }
{ * ameliorer le RAZ de l'EuroPSD     }
{  le timeout de 3s n'est pas raisonnable }
{ * dans la lecture de tableaux : demander le nombre d'adresses  lire et pas le nombre d'octets }
{   i.e. 4 octets par adresse                                                                    }
{ =================================== }
{ A NOTER                             }
{ sur PRISM, l'EuroPSD ne fonctionne pas avec l'adresse GPIB 7 ????  }
{ pas d'explicatio pour le moment     }
{ =================================== }
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,GPIB_CEC, EditData,math, BoiteNoire;

const
   AdresseEuroPSD=6;  { adresse gpib de la carte EuroPSD, au 22/01/2001: PRISM=6 et PAPOL=7}
   periodeframe=10000; { rsolution des frames de l'EuroPSD custom = 1ms pris par defaut pour l'instant}

type

    PSDarray65k=array[1..65536] of Cardinal;
    PSDarray16k=array[1..16384] of Cardinal;



  Function SendEuroPSD(s:string):string;
  procedure RAZEuroPSD;
  procedure checkTOF(var retard,largeurcanal:real; var nbrframe:integer);
  procedure param_TOF_var(var time:array of real;nbrframe:integer;var duree:real);
  procedure Lit_param_TOF_var(var time:array of real;nbrframe:integer;var duree:real);

  Procedure ReadArray65k(commande:string;var tab:PSDarray65k;lendem:longint; var test:boolean );
  Procedure ReadArray16k(commande:string;var tab:PSDarray16k;lendem:longint; var test:boolean );
  procedure litspectre(commande:string;moniteur:real;indexspectre:integer;comment:string);

  procedure EuroPSDInitsimple(decalage:integer);
  Procedure EuroPSDInitTOF(retard,largeurcanal:real;nbrframe:integer);
  Procedure EuroPSDInitTOF_Var(retard:real;nbrframe:integer);
  Procedure EuroPSDInitTOF_EROS(retard,largeurcanal:double;nbrframe:integer);

  procedure litspectre2D_PAPOL(decalage:integer;moniteur:real;indexspectre:integer;comment:string;polar:string);
  procedure litspectre_TOF16_PAPOL(indexspectre:integer;moniteur:real;comment:string);
  procedure litspectre_TOF8_PAPOL(indexspectreU,indexspectreD:integer;moniteurU,moniteurD:real;comment:string);
  procedure litspectre_TOF32_PAPOL(indexspectreU,indexspectreD:integer;moniteurU,moniteurD:real;comment:string);

  procedure litspectre2D_PRISM(moniteur:real;indexspectre,indexspectre1D:integer;comment:string;polar:string);
  procedure litspectre1D_Gabriel(moniteur:real;indexspectre1D:integer;comment:string;polar:string);
  procedure litspectre_PRISM_MS(commande:string;moniteur:real;indexspectre,indexspectre1D:integer;comment:string);

var

  tabtamp65k: PSDarray65k;
  tabtamp16k: PSDarray16k;
  D:array of integer; {stock la valeur des diviseurs en mode multiframe sur EuroPSD custom}
  tempsframe:array of real; {stock la dure de chaque frame en mode multiframe sur EuroPSD custom}

implementation

{************************************************
*
*                  Procedures de base
*
************************************************}


Function SendEuroPSD(s:string):string;
var s1:string;
begin
     GpibSend(AdresseEuroPSD,s);
     GpibRead(AdresseEuroPSD,s1,255);
     if (s1='?') or (s1='') then MessageDlg('ATTENTION : problme sur EUROPSD ', mtWarning,[mbOk],0);
     SendEuroPSD:=s1;
end;

{*************************************
RAZ la memoire de l'EuroPSD
time out de 3s}
procedure RAZEuroPSD;
var s:string;
    t,t0:real;

begin
t0:=(time-int(time))*86400;
GpibSend(AdresseEuroPSD,'Z');

repeat
      GpibRead(AdresseEuroPSD,s,255);
      t:=(time-int(time))*86400;
until (s='OK') or ((t-t0)>3)or ((t-t0)<0);

end;



{**********************************************
retourne les parrametres TOF dans la fourchette compatible
INPUT retard en s
      largeur canal en ms
      nbrframe
OUTPUT retard : nombre entier (mais format real) de 1/10 de s
       largeur canal en ms
      nbrframe}
procedure checkTOF(var retard, largeurcanal:real; var nbrframe:integer);

var l,E2:integer;
    t:int64; {largeur canal en 1/10 de s}

begin

{nombre de frame <16}
if nbrframe<2 then nbrframe:=2;
if nbrframe>32 then nbrframe:=32;

{4<retard<65535}
retard:=int(retard*10); {conversion en un entier de 1/10 de s}
if retard<4 then retard:=4;
if retard>65535 then retard:=65535;

{0.0004 ms<largeurcanal<429483 ms}
if largeurcanal<0.0004 then largeurcanal:=0.0004;
if largeurcanal>429483 then largeurcanal:=429483;


end;

{**********************************************
retourne les parametres TOF pour l'EUROPSD modifie (frame de duree variable)
INPUT time[0..15] : duree des frames en ms
      nbrframe
OUTPUT implicite : D tableau de diviseur pour l'EUROPSD
      time : duree des frames effectivement programmees en ms
      duree : duree total des frames programmees}
procedure param_TOF_var(var time:array of real;nbrframe:integer;var duree:real);
var i:integer;

begin
setlength(D,nbrframe);
setlength(tempsframe,nbrframe);
{nombre de frame <16
if nbrframe<2 then nbrframe:=2;
if nbrframe>16 then nbrframe:=16;     }

D[0]:=round(time[0]*power(10,4)/periodeframe);
time[0]:=round(D[0]*periodeframe*power(10,-4));
tempsframe[0]:=time[0];

duree:=time[0];
for i:=1 to nbrframe-1 do
    begin
    D[i]:=round((time[i]*power(10,4)-D[i-1])/(periodeframe-1));
    time[i]:=round((D[i]*(periodeframe-1)+D[i-1])*power(10,-4));
    tempsframe[i]:=time[i];
    duree:=duree+time[i];
    end;

end;
procedure Lit_param_TOF_var(var time:array of real;nbrframe:integer;var duree:real);
var i:integer;
    ligne1,ligne2,ligne3,ligne4:string;
    Dtemp:array[0..31]of integer;
begin
GpibSend(AdresseEuroPSD,'F');
GpibRead(AdresseEuroPSD,ligne1,255);
GpibRead(AdresseEuroPSD,ligne2,255);
GpibRead(AdresseEuroPSD,ligne3,255);
GpibRead(AdresseEuroPSD,ligne4,255);

for i:=0 to 7 do
    dtemp[i]:=strtoint(copy(ligne1,i*6+1,5));
for i:=0 to 7 do
    dtemp[i+8]:=strtoint(copy(ligne2,i*6+1,5));
for i:=0 to 7 do
    dtemp[i+16]:=strtoint(copy(ligne3,i*6+1,5));
for i:=0 to 7 do
    dtemp[i+24]:=strtoint(copy(ligne4,i*6+1,5));

time[0]:=Dtemp[0]*periodeframe*power(10,-4);
 tempsframe[0]:=time[0];

duree:=time[0];
for i:=1 to 31 do
    begin
    time[i]:=(Dtemp[i]*(periodeframe-1)+Dtemp[i-1])*power(10,-4);
    tempsframe[i]:=time[i];
    duree:=duree+time[i];
    end;

end;
{***************************************
*
* ensemble de procedures recuperation de data bas niveau
*
 *************************************** }


{***************************************
recupere un tableau de cardinal (65536 max )
INPUT: commande:commande GPIB
       lendem : nombre d'octet demand
OUTPUT: tab  PSDarray65k
        test: true si pas d'erreur observe
}
Procedure ReadArray65k(commande:string; var tab:PSDarray65k;lendem:longint; var test:boolean );
var len,status:longint;
    temp:string;
    i:integer;
begin
     temp:='UNL UNT MLA TALK0'+ inttostr(AdresseEuroPSD);
     GpibSend(AdresseEuroPSD,commande);
     GPIBtransmit(temp);      {arret de tous les dialogues sur GPIB}
     rarray32b (tab,lendem,len,status);
     GPIBtransmit ('UNL UNT');

     test:=(status=0) and (len=lendem);    {compare le nombre d'octet demand et recu}
end;

{***************************************
recupere un tableau de cardinal (16384 max) ,
INPUT: commande:commande GPIB
       lendem : nombre d'octet demand (ce nombre d'octets est egal  4 fois le nobre d'adresse) }
{OUTPUT: tab  PSDarray16k
        test: true si pas d'erreur observe }
Procedure ReadArray16k(commande:string;var tab:PSDarray16k;lendem:longint; var test:boolean );
var len,status:longint;
    temp:string;
    i:integer;
begin
     temp:='UNL UNT MLA TALK0'+ inttostr(AdresseEuroPSD);
     GpibSend(AdresseEuroPSD,commande);
     GPIBtransmit(temp);      {arret de tous les dialogues sur GPIB}
     rarray32b (tab,lendem,len,status);
     GPIBtransmit ('UNL UNT');

     test:=(status=0) and (len=lendem);    {compare le nombre d'octet demand et recu}
end;

{*************************************
procedure de chargement d'un spectre
lit les donnees et charge le spectre 2d specifi
modif du 19.09.01: la procedure s'adapte  la taille du spectre recepteur=taille du detecteur.

INPUT moniteur
      indexspectre: designe le numero du spectre a modifier
      comment:commentaire
      commande:commande GPIB

OUTPUT implicite : le spectre modifie}

procedure litspectre(commande:string;moniteur:real;indexspectre:integer;comment:string);
var i,j:integer;
    test:boolean;
    taille,X,Y:integer; {taille du spectre recepteur}

begin
X:=deuxd.spect[indexspectre].X;
Y:=deuxd.spect[indexspectre].Y;
taille:=trunc(X*Y*4); {taille en octet}
ReadArray16k(commande,tabtamp16k,taille,test);
deuxd.spect[indexspectre].moniteur:=moniteur;

for i:=0 to Y-1 do
    for j:=0 to X-1 do
    deuxd.spect[indexspectre].data[j,i,0].z:=tabtamp16k[Y*i+j+1];

deuxd.spect[indexspectre].comment:=comment;

end;

{*************************************
{ procedure de chargement d'un spectre pour detecteur PRISM }
{ lit les 16k donnees et charge le spectre 2d specifi      }
{ INPUT moniteur
      indexspectre: designe le numero du spectre a modifier
      comment:commentaire
      commande:commande GPIB
{ OUTPUT implicite : le spectre modifie}
procedure litspectre_PRISM(commande:string;moniteur:real;indexspectre2D,indexspectre1D:integer;comment:string);
var i,j,k:integer;
    test:boolean;
    v:integer;
begin
deuxd.spect[indexspectre2D].clear;
und.spect[indexspectre1D].clear;
ReadArray16k(commande,tabtamp16k,65536,test);
deuxd.spect[indexspectre2D].moniteur:=moniteur;
for i:=0 to 255 do begin
if i<=127 then k:=i+128 else k:=i-128;
v:=0;
    for j:=0 to 63 do begin
         if j>=32 then deuxd.spect[indexspectre2D].data[i,j,0].z:=tabtamp16k[256*(j-32)+k+1];
         if j<=31 then deuxd.spect[indexspectre2D].data[i,j,0].z:=tabtamp16k[256*(j+32)+k+1];
         v:=v+tabtamp16k[256*j+k+1];
end;
und.spect[indexspectre1D].data[i,0,0].z:=v;
und.spect[indexspectre1D].data[i,0,0].x:=-tan((i-130)/50)+(i-86)/400;
end;
deuxd.spect[indexspectre2D].comment:=comment;
end;

{ procedure de chargement d'un spectre pour detecteur PRISM }
{ lecture du detecter Gabriel                               }
{ lit les 256 donnees et charge le spectre 1d specifi      }
{ INPUT moniteur
      indexspectre: designe le numero du spectre a modifier
      comment:commentaire
      commande:commande GPIB
{ OUTPUT implicite : le spectre modifie}
procedure litspectre_Gabriel(commande:string;moniteur:real;indexspectre1D:integer;comment:string);
var i,j,k:integer;
    test:boolean;
    v:integer;
    index:byte;
    s:string;
begin
   index:=1;
   und.spect[indexspectre1D].clear;
   und.spect[indexSpectre1D].X:=256; { ARG... taille explicite ?!?!? }
   ReadArray16k(commande,tabtamp16k,1024,test); { A MODIFIER }
   v:=0; { v contient l'intgrale du spectre }
   for i:=0 to 255 do
   begin
      und.spect[indexspectre1D].data[i,0,0].x:=i;
      und.spect[indexspectre1D].data[i,0,0].z:=tabtamp16k[i];
      { cas ou il n'y a pas de coups dans le compteur }
      { ca permet d'utiliser l'echelle Log tout de meme }
      if und.spect[indexspectre1D].data[i,0,0].z=0 then und.spect[indexspectre1D].data[i,0,0].z:=0.01;
      v:=v+tabtamp16k[i];
   end;
   und.spect[indexspectre1D].comment:=comment;
   { TEMPORAURE }
{   deuxd.spect[index].clear;
   deuxd.spect[index].X:=128;deuxd.spect[index].Y:=128; { ARG... taille explicite ?!?!? }
{   s:=DataEditForm.lecturePSD.text;
   ReadArray65k(s,tabtamp65k,65536,test); { A MODIFIER }
{   ReadArray65k('T16384,32767',tabtamp65k,65536,test); { A MODIFIER }
{   ReadArray65k('T32768,49151',tabtamp65k,65536,test); { A MODIFIER }
{   ReadArray65k('T49152,65535',tabtamp65k,65536,test); { A MODIFIER }
{   v:=0; { v contient l'intgrale du spectre }
{   for i:=0 to 127 do
   for j:=0 to 127 do
   begin
      deuxd.spect[index].data[i,j,0].x:=i;
      deuxd.spect[index].data[i,j,0].z:=tabtamp65k[i+128*j];}
      { cas ou il n'y a pas de coups dans le compteur }
      { ca permet d'utiliser l'echelle Log tout de meme }
{      if und.spect[indexspectre1D].data[i,0,0].z=0 then und.spect[indexspectre1D].data[i,0,0].z:=0.01; }
{      v:=v+tabtamp65k[i+128*j]; }
{   end;
   deuxd.spect[index].comment:=comment;}
end;

{***************************************
*
* ensemble de procedures d'initialisation
*
 *************************************** }
{*************************************************
initialise euroPSD pour une acquisition XY
sans TOF
INPUT decalage: en fonction du cable utilis}

procedure EuroPSDInitsimple(decalage:integer);
var s,temp:string;

begin
s:=SendEuroPSD('D=01'); {mise en XY break}
s:=SendEuroPSD('#');    {routage off}
temp:='S=,'+inttostr(decalage);
s:=SendEuroPSD(temp); {Set du decalage du cable}
s:=SendEuroPSD('D=00'); {mise en XY marche}
RAZEuroPSD;    {reset}

end;

{*************************************************
initialise euroPSD pour une acquisition TOF

INPUT      retard: en s de 0.4 a 6553.5
           largeur canal: duree d'un canal en ms  de 0.0004 a 429483
           nbrcanal: nombre de frame <=16            }

Procedure EuroPSDInitTOF(retard,largeurcanal:real;nbrframe:integer);

var s:string;
    l,E2:integer;
    t:int64; {largeur canal en 1/10 de s}

begin
{verification des parametres}
checkTOF(retard,largeurcanal,nbrframe);

{determination du couple l/E2}
t:=round(largeurcanal*10000);
E2:=(t div 65535)+1;
if E2<2 then E2:=2;
l:=t div E2;

s:=SendEuroPSD('S=,0'); {il n'y a plus de decalage en mode TOF}
s:=SendEuroPSD('D=07'); {mise en: TOF, E2 diviseur, break}
s:=sendEuroPSD('P='+floattostr(retard)+','+inttostr(l)+','+inttostr(nbrframe)); {initialisation des parametres TOF}
s:=sendEuroPSD('E=0,'+inttostr(E2)); {coef de division}
s:=SendEuroPSD('#');    {routage off}
s:=SendEuroPSD('D=06'); {mise en: TOF, E2 diviseur, marche}
RAZEuroPSD;    {reset}


end;

{*************************************************
initialise euroPSD pour une acquisition TOF frame variable{EUROPSD version modifie}
{la procedure param_TOF doit avoir ete executee au prealable}
{INPUT      retard: en s de 0.4 a 6553.5
           nbrcanal: nombre de frame <=16            }

Procedure EuroPSDInitTOF_Var(retard:real;nbrframe:integer);

var s:string;
    i:integer;
    temp:real;


begin
temp:=50;
{verification des parametres}
checkTOF(retard,temp,nbrframe);
{initialisation du tableau de diviseurs}
for i:=1 to nbrframe do
    begin
    s:='F'+inttostr(i)+'='+inttostr(D[i-1]);
    sendEuroPSD(s);
    end;
s:=SendEuroPSD('S=,0'); {il n'y a plus de decalage en mode TOF}
s:=SendEuroPSD('D=07'); {mise en: TOF, E2 diviseur, break}
s:=sendEuroPSD('P='+floattostr(retard)+','+inttostr(periodeframe)+','+inttostr(nbrframe)); {initialisation des parametres TOF}

s:=SendEuroPSD('#');    {routage off}
s:=SendEuroPSD('D=06'); {mise en: TOF, E2 diviseur, marche}
RAZEuroPSD;    {reset}
end;

{ configuration de l'EuroPSD pour une acquisition en temps de vol }
{ sur le spectomtre EROS                                         }
{ INPUT : retard, en ms (typ. 1500s)                             }
{         largeurCanal, en ms (typ. 220s)                        }
{         nbrframe, nombre de canaux  (typ. 256)                  }
Procedure EuroPSDInitTOF_EROS(retard,largeurcanal:double;nbrframe:integer);
var s:string;
    l,E2:integer;
    t:int64; {largeur canal en 1/10 de s}
begin
{determination du couple l/E2}
   t:=round(largeurcanal*10000);
   E2:=(t div 65535)+1;
   if E2<2 then E2:=2;
   l:=t div E2;

s:=SendEuroPSD('S=,0'); {il n'y a pas de decalage en mode TOF}
s:=SendEuroPSD('D=07'); {mise en: TOF, E2 diviseur?? , break}
s:=sendEuroPSD('P='+floattostr(retard)+','+inttostr(l)+','+inttostr(nbrframe)); {initialisation des parametres TOF}
s:=sendEuroPSD('E=0,'+inttostr(E2)); {coef de division}
s:=SendEuroPSD('#');    {routage off}
s:=SendEuroPSD('D=06'); {mise en: TOF, E2 diviseur, marche}
RAZEuroPSD;    {reset}

end;


{***************************************
*
* ensemble de procedures recuperation de data haut niveau
*
 *************************************** }

{ ***************************************
lit un spectre 2D simple (avec le cable decale de 0, 3, 4 ou 5) UP ou DOWN en fonction de polar (lemo polar sur S2)

INPUT moniteur
      comment:commentaire
      indexspectre: designe le numero du spectre a modifier
      polar designe le set de data 

OUTPUT implicite: le spectre}
procedure litspectre2D_PAPOL(decalage:integer;moniteur:real;indexspectre:integer;comment:string;polar:string);
var s,temp:string;


begin

if polar='UP' then
   begin
   temp:='S=0,'+inttostr(decalage);
   s:=sendEuroPSD(temp);
   if decalage=0 then litspectre('T0,16383',moniteur,indexspectre,comment);
   if (decalage=3) or (decalage=4) then litspectre('X0,16383',moniteur,indexspectre,comment);
   if decalage=5 then litspectre('X0,4095',moniteur,indexspectre,comment);
   end;

if polar='DOWN' then
   begin
   temp:='S=2,'+inttostr(decalage);
   s:=sendEuroPSD(temp);
   if decalage=0 then litspectre('T0,16383',moniteur,indexspectre,comment);
   if decalage=3 then litspectre('X16384,32767',moniteur,indexspectre,comment); {l'instruction X ne prend pas en compte la configuration du Set mais uniquement le decalage d'@}
   if decalage=4 then litspectre('X0,16383',moniteur,indexspectre,comment); {en cable 4, le stockage d'un spectre est independant de la polar}
   if decalage=5 then litspectre('X4096,8191',moniteur,indexspectre,comment); {l'instruction X ne prend pas en compte la configuration du Set mais uniquement le decalage d'@}
   end;

end;


{***************************************************
recupere les donnees frame dans le cas d'une acquisition TOF 16 frames polarisation fixe (cable 4 bits)
et charge le spectre 3D indiqu par indexspectre
INPUT moniteur
      indexspectre
      comment:commentaire

OUTPUT implicite: le spectre 3D}

procedure litspectre_TOF16_PAPOL(indexspectre:integer;moniteur:real;comment:string);

var i,j,k,l:integer;
    test:boolean;
    testtamp:boolean;
    s:string;
    temp:real;

begin
{initialisation spectre}
troisd.spect[indexspectre].moniteur:=moniteur;
troisd.spect[indexspectre].comment:=comment;
{initialisation de l'echelle des temps}
for k:=0 to 15 do
    begin
    temp:=0;
    for l:=0 to k do
      temp:=temp+tempsframe[l]/1000;
    for i:=0 to 127 do
      for j:=0 to 127 do
        troisd.spect[indexspectre].data[i,j,k].lt:=temp;
    end;

{lecture du set 0 : lecture des 16 frames avec poid fort des adresses = 00}
s:=sendEuroPSD('S=0');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=0 to 31 do
    for i:=0 to 127 do
        for k:=0 to 15 do
            begin
            troisd.spect[indexspectre].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

{lecture du set 1 : lecture des 16 frames avec poid fort des adresses = 01}
s:=sendEuroPSD('S=1');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=32 to 63 do
    for i:=0 to 127 do
        for k:=0 to 15 do
            begin
            troisd.spect[indexspectre].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

{lecture du set 2 : lecture des 16 frames avec poid fort des adresses = 02}
s:=sendEuroPSD('S=2');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=64 to 95 do
    for i:=0 to 127 do
        for k:=0 to 15 do
            begin
            troisd.spect[indexspectre].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;


{lecture du set 3 : lecture des 16 frames avec poid fort des adresses = 03}
s:=sendEuroPSD('S=3');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=96 to 127 do
    for i:=0 to 127 do
        for k:=0 to 15 do
            begin
            troisd.spect[indexspectre].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

end;
{***************************************************
recupere les donnees frame dans le cas d'une acquisition TOF 8 frames polarisation alterne (cable 3 bits)
et charge les 16 spectres 2d
INPUT indexspextreU,indexspectreD: index des spectres 3D utiliss.
      moniteurs
      comment:commentaire

OUTPUT implicite: les spectres}

procedure litspectre_TOF8_PAPOL(indexspectreU,indexspectreD:integer;moniteurU,moniteurD:real;comment:string);
var i,j,k,l:integer;
    test:boolean;
    testtamp:boolean;
    s:string;
    temp:real;

begin
{initialisation spectre}
troisd.spect[indexspectreU].moniteur:=moniteurU;
troisd.spect[indexspectreD].moniteur:=moniteurD;
troisd.spect[indexspectreU].comment:=comment;
troisd.spect[indexspectreD].comment:=comment;

{initialisation de l'echelle des temps en s}
for k:=0 to 7 do
    begin
    temp:=0;
    for l:=0 to k do
      temp:=temp+tempsframe[l]/1000;
    for i:=0 to 127 do
      for j:=0 to 127 do
        begin
        troisd.spect[indexspectreU].data[i,j,k].lt:=temp;
        troisd.spect[indexspectreD].data[i,j,k].lt:=temp;
        end;
    end;

{lecture du set 0 : lecture des 8 frames avec poid fort des adresses = 00 neutron UP}
s:=sendEuroPSD('S=0');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=0 to 63 do
    for i:=0 to 127 do
        for k:=0 to 7 do
            begin
            troisd.spect[indexspectreU].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

{lecture du set 1 : lecture des 8 frames avec poid fort des adresses = 01 neutron UP}
s:=sendEuroPSD('S=1');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=64 to 127 do
    for i:=0 to 127 do
        for k:=0 to 7 do
            begin
            troisd.spect[indexspectreU].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

{lecture du set 2 : lecture des 8 frames avec poid fort des adresses = 02 neutron DOWN}
s:=sendEuroPSD('S=2');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=0 to 63 do
    for i:=0 to 127 do
        for k:=0 to 7 do
            begin
            troisd.spect[indexspectreD].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

{lecture du set 3 : lecture des 8 frames avec poid fort des adresses = 03 neutron DOWN}
s:=sendEuroPSD('S=3');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=64 to 127 do
    for i:=0 to 127 do
        for k:=0 to 7 do
            begin
            troisd.spect[indexspectreD].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;
end;

{***************************************************
recupere les donnees frame dans le cas d'une acquisition TOF 32 frames, polarisation alterne, detecteur 64*64 (cable 5 bits)
et charge les 2 spectres 3D specifis par indexspectre.
INPUT indexspextreU,indexspectreD: index des spectres 3D utiliss.
      moniteurs
      comment:commentaire

OUTPUT implicite: les spectres}

procedure litspectre_TOF32_PAPOL(indexspectreU,indexspectreD:integer;moniteurU,moniteurD:real;comment:string);
var i,j,k,l:integer;
    test:boolean;
    testtamp:boolean;
    s:string;
    temp:real;

begin
{initialisation spectre}
troisd.spect[indexspectreU].moniteur:=moniteurU;
troisd.spect[indexspectreD].moniteur:=moniteurD;
troisd.spect[indexspectreU].comment:=comment;
troisd.spect[indexspectreD].comment:=comment;

{initialisation de l'echelle des temps en s}
for k:=0 to 31 do
    begin
    temp:=0;
    for l:=0 to k do
      temp:=temp+tempsframe[l]/1000;
    for i:=0 to 63 do
      for j:=0 to 63 do
        begin
        troisd.spect[indexspectreU].data[i,j,k].lt:=temp;
        troisd.spect[indexspectreD].data[i,j,k].lt:=temp;
        end;
    end;

{lecture du set 0 : lecture des 8 frames avec poid fort des adresses = 00 neutron UP}
s:=sendEuroPSD('S=0');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=0 to 31 do
    for i:=0 to 63 do
        for k:=0 to 31 do
            begin
            troisd.spect[indexspectreU].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

{lecture du set 1 : lecture des 8 frames avec poid fort des adresses = 01 neutron UP}
s:=sendEuroPSD('S=1');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=32 to 63 do
    for i:=0 to 63 do
        for k:=0 to 31 do
            begin
            troisd.spect[indexspectreU].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

{lecture du set 2 : lecture des 8 frames avec poid fort des adresses = 02 neutron DOWN}
s:=sendEuroPSD('S=2');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=0 to 31 do
    for i:=0 to 63 do
        for k:=0 to 31 do
            begin
            troisd.spect[indexspectreD].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;

{lecture du set 3 : lecture des 8 frames avec poid fort des adresses = 03 neutron DOWN}
s:=sendEuroPSD('S=3');
ReadArray65k('T0,65535',tabtamp65k,262144,testtamp);

l:=1;
for j:=32 to 63 do
    for i:=0 to 63 do
        for k:=0 to 31 do
            begin
            troisd.spect[indexspectreD].data[i,j,k].z:=tabtamp65k[l];
            l:=l+1;
            end;
end;

{ ***************************************
lit un spectre 2D simple (avec le cable decale de 0 ou 4) UP ou DOWN en fonction de polar (lemo polar sur S2)
INPUT moniteur
      comment:commentaire
      indexspectre: designe le numero du spectre a modifier
      polar designe le set de data

OUTPUT implicite: le spectre}
procedure litspectre2D_PRISM(moniteur:real;indexspectre,indexspectre1D:integer;comment:string;polar:string);
var s,temp:string;
begin

if polar='UP' then
   begin
   temp:='S=0,0'; { up-up et pas de dcalage }
   s:=sendEuroPSD(temp)
   end;

if polar='DOWN' then
   begin
   temp:='S=2,0'; { down-down et pas de dcalage }
   s:=sendEuroPSD(temp)
   end;
{ litspectre_PRISM('T0,16383',moniteur,indexspectre,indexspectre1D,comment); version Gabriel je suppose }
litspectre_PRISM_MS('X0,65535',moniteur,indexspectre,indexspectre1D,comment); { version microstrips }

end;

{ ***************************************
lit un spectre 1D simple (avec le cable decale de 0 ou 4) UP ou DOWN en fonction de polar (lemo polar sur S2)
INPUT moniteur
      comment:commentaire
      indexspectre: designe le numero du spectre a modifier
      polar designe le set de data
OUTPUT implicite: le spectre}
procedure litspectre1D_Gabriel(moniteur:real;indexspectre1D:integer;comment:string;polar:string);
var s,temp:string;
begin
 if polar='UP' then
   begin
   temp:='S=0,0'; { up-up et pas de dcalage }
   s:=sendEuroPSD(temp)
   end;
 if polar='DOWN' then
   begin
   temp:='S=2,0'; { down-down et pas de dcalage }
   s:=sendEuroPSD(temp)
   end;
 { ******* changer les adresses ********* }
{ litspectre_Gabriel('T16257,16384',moniteur,indexspectre1D,comment); }
{  litspectre_Gabriel('T16129,16256',moniteur,indexspectre1D,comment); }
  litspectre_Gabriel('T16129,16384',moniteur,indexspectre1D,comment);
end;

{ ***** version bricolage pour lecture sur le rack prototype des MicroStrips }
procedure litspectre_PRISM_MS(commande:string;moniteur:real;indexspectre,indexspectre1D:integer;comment:string);
var i,j,k,l:integer;
    test:boolean;
    v:integer;
    va,vb,vc:array[0..512] of double;
    index,index2,idx:integer;
    total:integer;
    s:string;
begin
index:=und.indexof('VA');
index2:=und.indexof('VB');
for i:=0 to 512 do va[i]:=0;
for i:=0 to 512 do vb[i]:=0;
for i:=0 to 512 do vc[i]:=0;
deuxd.spect[indexspectre].clear;
und.spect[indexspectre1D].clear;
und.spect[index].clear;
und.spect[index2].clear;
ReadArray65k(commande,tabtamp65k,262144,test);
deuxd.spect[indexspectre].data[0,0,0].mon:=moniteur;
for i:=0 to 255 do begin { why was it 180 instead of 255??? }
v:=0;
{l:= (i and $3F)*4+(i and $C0) div 64;}
l:=i;
    for j:=0 to 255 do begin
{k:= (j and $3F)*4+(j and $C0) div 64;}
k:=j;
     {if j>=128 then deuxd.spect[indexspectre].data[l,k].z:=tabtamp65k[256*(j-128)+i+1];
     if j<=127 then deuxd.spect[indexspectre].data[l,k].z:=tabtamp65k[256*(j+128)+i+1];}
     deuxd.spect[indexspectre].data[k,l,0].x:=k;
     deuxd.spect[indexspectre].data[k,l,0].y:=l;
     deuxd.spect[indexspectre].data[k,l,0].z:=tabtamp65k[256*j+i+1];
     va[l]:=va[l]+deuxd.spect[indexspectre].data[k,l,0].z;
     vb[k]:=vb[k]+deuxd.spect[indexspectre].data[k,l,0].z;
    end;
end;
total:=0;
for i:=0 to 255 do begin
if va[i]<1 then va[i]:=0;
if vb[i]<1 then vb[i]:=0;
und.spect[indexspectre1D].data[i,0,0].z:=va[i];
und.spect[index].data[i,0,0].z:=vb[i];
total:=total+round(va[i]);
und.spect[indexspectre1D].data[i,0,0].x:=i;
und.spect[index].data[i,0,0].x:=i;
end;
s:='TOTAL : '+inttostr(total)+';';
boitnoir.receivetext(s);
deuxd.spect[indexspectre].comment:=comment;

end;

end.
